//
//  DummyViewController.m
//  SwiftCam
//
//  Created by Hirohito Kato on 2014/11/15.
//  Copyright (c) 2015 SwiftBook. All rights reserved.
//
@import AVFoundation;
@import Social;
#import "DummyViewController.h"
#import "LiveView.h"

@interface DummyViewController () <AVCaptureFileOutputRecordingDelegate>
@property (nonatomic, weak) IBOutlet LiveView *liveView;
@property (nonatomic, weak) IBOutlet UIButton *recordButton;

@property (nonatomic) AVCaptureSession *captureSession;
@property (nonatomic) AVCaptureMovieFileOutput *movieOut;
@property (nonatomic) AVCaptureStillImageOutput *photoOut;

- (NSURL *)movieURL:(NSString *)filename;
- (BOOL)removeFile:(NSURL *)fileURL;
@end

@implementation DummyViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self setupCaptureSession];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [_captureSession startRunning];
}

- (void)viewWillDisappear:(BOOL)animated {
    [_captureSession stopRunning];
    [super viewWillDisappear:animated];
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setupCaptureSession {
    NSError *error;

    _captureSession = [[AVCaptureSession alloc] init];
    _movieOut = [[AVCaptureMovieFileOutput alloc] init];
    _photoOut = [[AVCaptureStillImageOutput alloc] init];

    // 1920x1080で撮影するプリセットを指定
    _captureSession.sessionPreset = AVCaptureSessionPreset1280x720;

    // カメラのデフォルトデバイスを取得（背面カメラ）
    AVCaptureDevice *device
    = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];

    // デバイスをキャプチャセッションにつなげるためのインプットを取得
    AVCaptureDeviceInput *deviceIn
    = [AVCaptureDeviceInput deviceInputWithDevice:device
                                            error:&error];

    // インプットが接続できるか確認した上でキャプチャセッションに接続
    if ([_captureSession canAddInput:deviceIn]) {
        [_captureSession addInput:deviceIn];
    }

    // カメラ出力をムービーファイルとして書き込むアウトプットを接続
    if ([_captureSession canAddOutput:_movieOut]) {
        [_captureSession addOutput:_movieOut];
    }
    // カメラ出力から写真を撮影するアウトプットを接続
    if ([_captureSession canAddOutput:_photoOut]) {
        [_captureSession addOutput:_photoOut];
    }

    // ライブビューに表示対象のキャプチャセッションを指定
    _liveView.session = _captureSession;

    // 動画の解像度に関わらず、写真を最大解像度で撮影するよう指定
    _photoOut.highResolutionStillImageOutputEnabled = YES;

}

- (IBAction)record:(id)sender {
    if (!_movieOut.recording) {
        // 録画の開始
        // ムービーファイルのURLを作成
        NSURL *fileURL = [self movieURL:@"movie.MOV"];
        [self removeFile:fileURL];

        // 作成したURLを使って録画を開始
        [_movieOut startRecordingToOutputFileURL:fileURL
                               recordingDelegate:self];
    } else {
        // 録画を終了
        [_movieOut stopRecording];

        // ファイルが完成するまで、録画ボタンを無効化しておく
        // （重複して書き込まれるのを防ぐ）
        UIButton *button = sender;
        button.enabled = NO;
    }
}

- (NSURL *)movieURL:(NSString *)filename {
    NSString *filePath = [NSTemporaryDirectory() stringByAppendingString:filename];

    return [NSURL fileURLWithPath:filePath];
}
- (BOOL)removeFile:(NSURL *)fileURL {
    BOOL success = NO;

    // カメラロールに保存していない古いファイルを削除
    NSString *filepath = fileURL.path;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if ([fileManager fileExistsAtPath:filepath]) {
        success = [fileManager removeItemAtPath:filepath error:nil];
    }
    return success;
}

- (void)captureOutput:(AVCaptureFileOutput *)output
didFinishRecordingToOutputFileAtURL:(NSURL *)outputFileURL
      fromConnections:(NSArray *)connections
                error:(NSError *)error {

    // カメラロールに保存
    UISaveVideoAtPathToSavedPhotosAlbum(outputFileURL.path,
        self, @selector(image:didFinishSavingWithError:contextInfo:), nil);
}

- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error
  contextInfo:(void*)info {

    // ボタンの状態を戻す
    _recordButton.enabled = YES;

    NSLog(@"ムービーをカメラロールに保存しました");
}

- (IBAction)takeAndShare:(id)sender {
    // カメラに繋がっているコネクションを選択
    AVCaptureConnection *connection =
        [_photoOut connectionWithMediaType:AVMediaTypeVideo];

    // コネクション経由で撮影
    [_photoOut captureStillImageAsynchronouslyFromConnection:connection
                                           completionHandler:
     ^(CMSampleBufferRef sampleBuffer, NSError *error) {

         if (error == nil) {
             // 得られたデータをSNSに投稿
             [self shareImageToSNS:sampleBuffer];
         }
     }];
}

- (void)shareImageToSNS:(CMSampleBufferRef)sampleBuffer {
    // 撮影データをJPEG画像に変換してからUIImageに変換する
    NSData *data = [AVCaptureStillImageOutput
                    jpegStillImageNSDataRepresentation:sampleBuffer];
    UIImage *image = [UIImage imageWithData:data];

    // Twitter投稿用のビューコントローラを作成
    SLComposeViewController *vc = [SLComposeViewController
        composeViewControllerForServiceType:SLServiceTypeTwitter];
    [vc addImage:image];
    [vc setInitialText:@"#SwiftCam"];
    vc.completionHandler = ^(SLComposeViewControllerResult result) {
        NSLog(@"SNSへの投稿処理が終わりました。");
    };

    // 投稿画面を表示
    [self presentViewController:vc animated:YES completion:nil];
}
@end
