//
//  ViewController.swift
//  SwiftCam
//
//  Created by Hirohito Kato on 2015/09/22.
//  Copyright (c) 2015 SwiftBook. All rights reserved.
//

import UIKit
import AVFoundation
import Social

class ViewController: UIViewController, AVCaptureFileOutputRecordingDelegate {

    override func viewDidLoad() {
        super.viewDidLoad()
        setupCaptureSession()
    }

    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)

        // ビュー表示と同時にキャプチャセッションもスタート
        captureSession.startRunning()
    }

    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)

        // キャプチャセッションの停止
        captureSession.stopRunning()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    @IBOutlet weak var liveView: LiveView!
    @IBOutlet weak var recordButton: UIButton!

    @IBAction func record(sender: AnyObject) {
        if !movieOut.recording {
            // 録画の開始
            // ムービーファイルのURLを作成
            let fileURL = movieURL("movie.MOV")
            self.removeFile(fileURL)

            // 作成したURLを使って録画を開始
            movieOut.startRecordingToOutputFileURL(fileURL,
                recordingDelegate: self)
        } else {
            // 録画を終了
            movieOut.stopRecording()

            // ファイルが完成するまで、録画ボタンを無効化しておく
            // （重複して書き込まれるのを防ぐ）
            let button = sender as! UIButton
            button.enabled = false;
        }
    }

    @IBAction func takeAndShare(sender: AnyObject) {
        // カメラに繋がっているコネクションを選択
        let connection = photoOut.connectionWithMediaType(AVMediaTypeVideo)

        // コネクション経由で撮影
        photoOut.captureStillImageAsynchronouslyFromConnection(connection)
            { sampleBuffer, error in

            if error == nil {
                // 得られたデータをSNSに投稿
                self.shareImageToSNS(sampleBuffer)
            }
        }
    }

    let captureSession = AVCaptureSession()
    let movieOut = AVCaptureMovieFileOutput()
    let photoOut = AVCaptureStillImageOutput()

    func setupCaptureSession() {

        // 720pで撮影するプリセットを指定
        captureSession.sessionPreset = AVCaptureSessionPreset1280x720

        // カメラのデフォルトデバイスを取得（背面カメラ）
        // ※iPod touch 5th 16GBだけは前面カメラがデフォルト
        let device: AVCaptureDevice!
            = AVCaptureDevice.defaultDeviceWithMediaType(AVMediaTypeVideo)

        // デバイスをキャプチャセッションにつなげるためのインプットを取得
        let deviceIn: AVCaptureDeviceInput!
        do {
            deviceIn = try AVCaptureDeviceInput(device: device)
        } catch _ as NSError {
            deviceIn = nil
        }

        // インプットが接続できるか確認した上でキャプチャセッションに接続
        if captureSession.canAddInput(deviceIn) {
            captureSession.addInput(deviceIn)
        }

        // カメラ出力をムービーファイルとして書き込むアウトプットを接続
        if captureSession.canAddOutput(movieOut) {
            captureSession.addOutput(movieOut)
        }
        // カメラ出力から写真を撮影するアウトプットを接続
        if captureSession.canAddOutput(photoOut) {
            captureSession.addOutput(photoOut)
        }

        // ライブビューに表示対象のキャプチャセッションを指定
        liveView.session = captureSession
        
        // 動画の解像度に関わらず、写真を最大解像度で撮影するよう指定
        // （2015/8時点でiPhone6/6 plusのみ有効）
        photoOut.highResolutionStillImageOutputEnabled = true
    }

    //MARK: 動画
    private func movieURL(filename: String) -> NSURL {
        let filepath = (NSTemporaryDirectory() as NSString)
            .stringByAppendingPathComponent(filename)

        return NSURL(fileURLWithPath: filepath)
    }
    private func removeFile(fileURL: NSURL) -> Bool {

        guard let filepath = fileURL.path else {
            return false
        }

        var success = false

        // カメラロールに保存していない古いファイルを削除
        let fileManager = NSFileManager.defaultManager()
        if fileManager.fileExistsAtPath(filepath) {
            do {
                try fileManager.removeItemAtPath(filepath)
                success = true
            } catch _ {
            }
        }
        return success
    }

    //MARK: - AVCaptureFileOutputRecordingDelegateプロトコルのメソッド
    internal func captureOutput(captureOutput: AVCaptureFileOutput!,
        didFinishRecordingToOutputFileAtURL outputFileURL: NSURL!,
        fromConnections connections: [AnyObject]!, error: NSError!) {

            // カメラロールに保存
            UISaveVideoAtPathToSavedPhotosAlbum(outputFileURL.path!,
                self, "image:didFinishSavingWithError:contextInfo:", nil);
    }

    //MARK: 写真
    func shareImageToSNS(sbuf:CMSampleBuffer) {
        // 撮影データをJPEG画像に変換してからUIImageに変換する
        let data = AVCaptureStillImageOutput.jpegStillImageNSDataRepresentation(sbuf)
        let image = UIImage(data: data)

        // Twitter投稿用のビューコントローラを作成
        let vc = SLComposeViewController(forServiceType: SLServiceTypeTwitter)
        vc.addImage(image)
        vc.setInitialText("#SwiftCam")
        vc.completionHandler = { result in
            print("SNSへの投稿処理が終わりました。")
        }

        // 投稿画面を表示
        self.presentViewController(vc, animated: true, completion: nil)
    }

    func image(image:UIImage, didFinishSavingWithError error:NSError,
        contextInfo info:UnsafeMutablePointer<Void>) {

        // ボタンの状態を戻す
        self.recordButton.enabled = true

        print("ムービーをカメラロールに保存しました")
    }


}
